#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import pprint
import algobroker
from algobroker import Broker
from cryptoexchange import bitmex
import sys
import traceback

class BrokerBitmex(Broker):
    def __init__(self):
        Broker.__init__(self, "broker_bitmex")
        self.api = None
    def process_data(self, data):
        if self.api == None:
            self.error("keys not initialized")
            self.error(pprint.pformat(data))
            return
        cmd = data.get('cmd', "None")
        self.debug("processing data command %s" % cmd)
        if cmd == 'order':
            self.api.place_order(data.get('quantity', None),
                                 data.get('symbol', None),
                                 data.get('price', None))
        elif cmd == 'cancel':
            self.debug("cancelling order")
            self.api.cancel(data.get('orderID', None))
            self.debug("orders cancelled")
        elif cmd == 'cancel_all':
            self.debug("getting order list")
            orders = self.api.open_orders()
            self.debug("cancelling orders")
            for i in orders:
                self.api.cancel(i.get('orderID', None))
            self.debug("orders cancelled")
        elif cmd == "report_all":
            orders = self.api.open_orders()
            self.info(pprint.pformat(data))
        elif cmd == 'position':
            self.info(pprint.pformat(self.api.position()))
        else:
            raise RuntimeError("unknown data %s" % cmd)
    def process_control(self, data):
        if algobroker.Broker.process_control(self, data):
            return True
        cmd = data.get("cmd", "None")
        if cmd == "auth":
            self.info("received auth message")
            base_url = data.get('base_url', None)
            login = data.get('login', None)
            password = data.get('password', None)
            otpToken = data.get('otpToken', None)
            apiKey = data.get('apiKey', None)
            apiSecret = data.get('apiSecret', None)
            orderIDPrefix = data.get('orderIDPrefix', 'algo_bitmex_')
            self.api = bitmex.BitMEX(base_url,
                                     login,
                                     password,
                                     otpToken,
                                     apiKey,
                                     apiSecret,
                                     orderIDPrefix)
            try:
                self.api.authenticate()
                self.debug("get positions")
                self.debug(pprint.pformat(self.api.position()))
            except:
                self.error("Authentication error")
                self.api=None
        else:
            raise RuntimeError("unknown command %s" % cmd)

if __name__ == "__main__":
    bp = BrokerBitmex()
    bp.run()



# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import logging
import zmq
import msgpack
import time
import traceback


def logger(s : str):
    logger = logging.getLogger(s)
    logger.setLevel(logging.INFO)
    ch = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    formatter.converter = time.gmtime
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    return logger

loglevels = {
    "CRITICAL" : logging.CRITICAL,
    "ERROR" : logging.ERROR,
    "WARNING" : logging.WARNING,
    "INFO" : logging.INFO,
    "DEBUG" : logging.DEBUG,
    "NOTSET" : logging.NOTSET
    }

ports = {
    "data" : {
    "dispatcher" : "tcp://127.0.0.1:5557",
    "broker_plivo" : "tcp://127.0.0.1:5558",
    "strategy_alert" : "tcp://127.0.0.1:5559",
    "ticker_yahoo" : "tcp://127.0.0.1:5560",
    "ticker_bitfutures" : "tcp://127.0.0.1:5561",
    "broker_bitmex" : "tcp://127.0.0.1:5562",
    "ticker_bravenewcoin" : "tcp://127.0.0.1:5563",
    "strategy_xbt_close" : "tcp://127.0.0.1:5564",
    "broker_desk_alert" : "tcp://127.0.0.1:5565",
    "broker_twilio" : "tcp://127.0.0.1:5566",
    "broker_web" : "tcp://127.0.0.1:5567",
    },
    "control" : {
    "dispatcher" : "tcp://127.0.0.1:5577",
    "broker_plivo" : "tcp://127.0.0.1:5578",
    "strategy_alert" : "tcp://127.0.0.1:5579",
    "ticker_yahoo" : "tcp://127.0.0.1:5580",
    "ticker_bitfutures" : "tcp://127.0.0.1:5581",
    "broker_bitmex" : "tcp://127.0.0.1:5582",
    "ticker_bravenewcoin" : "tcp://127.0.0.1:5583",
    "strategy_xbt_close" : "tcp://127.0.0.1:5584",
    "broker_desk_alert" : "tcp://127.0.0.1:5585",
    "broker_twilio" : "tcp://127.0.0.1:5586",
    "broker_web" : "tcp://127.0.0.1:5587",
    }
    }

def send(name, data):
    context = zmq.Context()
    for i in data:
        socket = context.socket(zmq.PUSH)
        socket.connect(ports[name][i['dest']])
        socket.send(msgpack.packb(i))



class AlgoObject(object):
    def __init__(self, name : str, socket_type):
        self._logger = logger(name)
        self._context = zmq.Context()
        self._poller = zmq.Poller()
        self._data_socket = self.socket(socket_type)

        self._control_socket = self.socket(zmq.PULL)
        self._control_socket.bind(ports['control'][name])
        self._poller.register(self._data_socket,
                              zmq.POLLIN)
        self._poller.register(self._control_socket,
                              zmq.POLLIN)
        self.info("starting %s" % name)
        self.timeout = None
    def socket(self, socket_type):
        return self._context.socket(socket_type)
    def recv_data(self):
        return msgpack.unpackb(self._data_socket.recv(),
                               encoding='utf-8')
    def recv_control(self):
        return msgpack.unpackb(self._control_socket.recv(),
                               encoding='utf-8')
    def debug(self, s):
        self._logger.debug(s)
    def info(self, s):
        self._logger.info(s)
    def error(self, s):
        self._logger.error(s)
    def warning(self, s):
        self._logger.warning(s)
    def set_logger_level(self, level):
        self._logger.setLevel(level)
    def process_data(self, data):
        raise NotImplementedError
    def process_control(self, data):
        self.debug("received control message")
        if data.get('cmd', None) == 'loglevel':
            if data.get('level', None) in loglevels:
                self.set_logger_level(loglevels[data['level']])
                self.info(("Setting loglevel to %s", data['level']))
                return True
    def run_once(self):
        pass
    def run(self):
        while True:
            try:
                socks = dict(self._poller.poll(self.timeout))
            except KeyboardInterrupt:
                break
            try:
                if self._control_socket in socks:
                    control = self.recv_control()
                    self.process_control(control)
                if self._data_socket in socks:
                    data = self.recv_data()
                    self.process_data(data)
                self.run_once()
            except:
                self.error("error processing control message")
                self.error(traceback.format_exc())

class Strategy(AlgoObject):
    def __init__(self, name, tickers):
        AlgoObject.__init__(self, name, zmq.SUB)
        for i in tickers:
            self._data_socket.connect(ports['data'][i])
        self._data_socket.setsockopt(zmq.SUBSCRIBE, b'')
        self._action_socket = self.socket(zmq.PUSH)
        self._action_socket.connect(ports['data']['dispatcher'])
    def send_action(self, message):
        self._action_socket.send(msgpack.packb(message))
    def send_control(self, to, message):
        socket = self._context.socket(zmq.PUSH)
        socket.connect(ports['control'][to])
        socket.send(msgpack.packb(message))

class Broker(AlgoObject):
    def __init__(self, name):
        AlgoObject.__init__(self, name, zmq.PULL)
        self._data_socket.bind(ports['data'][name])
    def process_control(self, data):
        return AlgoObject.process_control(self, data)

class Ticker(AlgoObject):
    def __init__(self, name):
        AlgoObject.__init__(self, name, zmq.PUB)
        self._data_socket.bind(ports['data'][name])
        self.timeout = 30000
        self.quotes = {}
    def run_once(self):
        self.debug("running loop function")
        self.get_quotes()
        self.send_quotes()
    def process_control(self, data):
        return AlgoObject.process_control(self, data)
    def send_quotes(self):
        self.debug("Sending quotes")
        self.send_data(self.quotes)
    def send_data(self, message):
        self._data_socket.send(msgpack.packb(message))
    def test(self):
        self.get_quotes()
        socket = self._context.socket(zmq.PUSH)
        socket.bind(ports['data']['dispatcher'])
        message = { 'cmd' : 'log',
                    'item' : self.quotes }
        self._logger.debug("Sending data")
        socket.send(msgpack.packb(message))



#!/usr/bin/env python3
# http://flask.pocoo.org/snippets/116/

import zmq.green as zmq
from flask import Flask, send_from_directory, Response, request
import flask
import algobroker
from io import StringIO
from gevent.queue import Queue
import sys
import time
from gevent.wsgi import WSGIServer
import gevent

app = Flask(__name__, static_url_path='')

class BrokerWeb(algobroker.Broker):
    def __init__(self):
        algobroker.Broker.__init__(self, "broker_web")
    def process_data(self, data):
        self.info(data)

subscriptions = []
# SSE "protocol" is described here: http://mzl.la/UPFyxY
class ServerSentEvent(object):
    def __init__(self, data):
        self.data = data
        self.event = None
        self.id = None
        self.desc_map = {
            self.data : "data",
            self.event : "event",
            self.id : "id"
            }
    def encode(self):
        if not self.data:
            return ""
        lines = ["%s: %s" % (v, k)
                 for k, v in self.desc_map.items() if k]
        return "%s\n\n" % "\n".join(lines)

@app.route("/")
def hello():
    return app.send_static_file('broker_web.html')

@app.route("/test-data")
def testdata():
    return flask.jsonify({"records" : [{"Name" : "foo",
                           "Country" : "bar"},
                          {"Name" : "foo1",
                           "Country" : "bar1"}]})

@app.route("/inject-control", methods=['GET', 'POST'])
def injectControl():
    algobroker.send("control",
                    request.json)
    return "OK"

@app.route("/inject-data", methods=['GET', 'POST'])
def injectData():
    algobroker.send("data",
                    request.json)
    return "OK"

@app.route("/desk-alert")
def deskalert():
    algobroker.send("data",
                    [{"dest" : "broker_desk_alert",
                     "cmd" : "alert",
                     "type" : "desk",
                     "alert" : "high"}])
    return "alert"

@app.route("/debug")
def debug():
    return "Currently %d subscriptions" % len(subscriptions)

@app.route("/publish")
def publish():
    msg = str(time.time())
    for sub in subscriptions[:]:
        sub.put(msg)
    return "OK"

@app.route("/subscribe")
def subscribe():
    def gen():
        q = Queue()
        subscriptions.append(q)
        try:
            while True:
                result = q.get()
                ev = ServerSentEvent(str(result))
                yield ev.encode()
        except GeneratorExit: # Or maybe use flask signals
            subscriptions.remove(q)
    return Response(gen(), mimetype="text/event-stream")



if __name__ == "__main__":
    bw = BrokerWeb()
    g = gevent.Greenlet.spawn(bw.run)
    app.debug=True
    http_server = WSGIServer(('', 5000), app)
    http_server.serve_forever()
    # Then visit http://localhost:5000 to subscribe
    # and send messages by visiting http://localhost:5000/publish



#!/usr/bin/python3
import algobroker
algobroker.send("control",
                [
    {"dest": "ticker_yahoo",
     "cmd" : "set",     
     "assets" : ["3888.HK", "0700.HK", "0388.HK"]
     },
    {"dest": "broker_bitmex",
     "cmd" : "loglevel",
     "level" : "DEBUG"
    },
    {"dest": "ticker_bitfutures",
     "cmd" : "loglevel",
     "level" : "DEBUG"
    },
    {"dest": "ticker_bitfutures",
     "cmd" : "set",
     "exchanges" : ["bitmex"]
    },
    {"dest": "strategy_alert",
     "cmd" : "set",
     "limits" : {
    "3888.HK" : [ 14.8, 15.5],	
    "0700.HK" : [ 125.0, 130.0],
    "0388.HK" : [ 175.0, 180.0]
    }
    }
    ])




#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import pprint
import algobroker
from algobroker import Broker
import twilio

class BrokerTwilio(Broker):
    def __init__(self):
        Broker.__init__(self, "broker_twilio")
        self.api = None
        self.src_number = None
        self.dst_number = None
    def process_data(self, data):
        if self.api == None or \
           self.src_number == None or \
           self.dst_number == None:
            self.error("keys not initialized")
        if (data['cmd'] == "alert" and \
            data['type'] == 'sms'):
            params = {
                'from' : self.src_number,
                'to' : self.dst_number[data['to']],
                'body' : data['text'],
                }
            self.debug(pprint.pformat(params))
            response = self.api.messages.create(**params)
            self.debug(pprint.pformat(str(response)))
        else:
            self.error("unknown item")
            self.error(pprint.pformat(data))
    def process_control(self, data):
        algobroker.Broker.process_control(self, data)
        if data.get('cmd', None) == "auth":
            self.auth_id = data['TWILIO_AUTH_ID']
            self.auth_token = data['TWILIO_AUTH_TOKEN']
            self.api = twilio.rest.TwilioRestClient(self.auth_id,
                                                    self.auth_token)
            self.src_number = data['src_number']
            self.dst_number = data['dst_number']

if __name__ == "__main__":
    bp = BrokerTwilio()
    bp.run()



#!/usr/bin/python3
import algobroker
algobroker.send("control",
                [
    {
    "dest": "broker_bitmex",
    "cmd" : "loglevel",
    "level" : "DEBUG"
    }])

algobroker.send("data",
                [
    {
    "dest": "broker_bitmex",
    "cmd" : "position"
    },
    {
    "dest": "broker_bitmex",
    "cmd" : "cancel_all"
    },
    ])

"""
    {
    "dest": "broker_bitmex",
    "cmd" : "order",
    "quantity" : -10,
    "symbol" : "XBT7D",
    "price" : 241.00
    }
"""




#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import time
import algobroker
from cryptoexchange import bitfutures
import pprint

class BitfuturesTicker(algobroker.Ticker):
    def __init__(self):
        algobroker.Ticker.__init__(self, "ticker_bitfutures")
        self.exchanges = []
        self.time_limits = {}
        self.state = {}
        self.prev_state = {}
    def get_quotes(self):
        self.debug("getting quotes")
        data = bitfutures.get_data(self.exchanges)
        try:
            data = data['futures']
            for i in self.exchanges:
                if i in data:
                    j = data[i]
                    self.debug(j)
                    for k in zip(j['dates'],
                                 j['asks'],
                                 j['bids'],
                                 j['last'],
                                 j['contract']):
                        tag = k[4] + "." + k[0].replace("-", "") + "." + i
                        self.quotes[tag] = {
                            "ask" : float(k[1]) if k[1] != None else None,
                            "bid" : float(k[2]) if k[2] != None else None,
                            "last" : float(k[3]) if k[3] != None else None,
                            }
            self.debug(self.quotes)
        except OSError:
            self.error("Network Error")
            time.sleep(60)
    def process_control(self, data):
        algobroker.Ticker.process_control(self, data)
        if data['cmd'] == "set":
            if 'exchanges' in data:
                self.debug("setting exchange list")
                self.debug(pprint.pformat(data['exchanges']))
                self.exchanges = data['exchanges']
        elif data['cmd'] == "test":
            self.debug("received test message")
            self.test()

if __name__ == "__main__":
    yq = BitfuturesTicker()
    yq.run()



#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License
import my_path
import time
import algobroker
import pprint

class StrategyXbtClose(algobroker.Strategy):
    def __init__(self):
        algobroker.Strategy.__init__(self, "strategy_xbt_close",
                                     ['ticker_bravenewcoin'])
        self.range = 0.01
        self.xbt_initial_price = None
        self.xbt_current_price = None
        self.active = True
        self.set_logger_level("DEBUG")
        self.send_control("ticker_bravenewcoin",
                          {"cmd" : "set",
                           "assets" : ['btc_usd_24hr']})
    def send_cancel(self):
        msg = "XBT orders cancelled"
        self.send_action({'cmd' : 'alert',
                          'type' : 'sms',
                          'to' : 'trader1',
                          'text' : msg})
        self.send_action({'cmd' : 'cancel_all',
                          'broker' : 'bitmex'})
        self.active = False
        self.xbt_initial_price = None
    def process_control(self, data):
        if algobroker.Strategy.process_control(self, data):
            return
        if data['cmd'] == "set":
            if 'range' in data:
                self.range = data['limits']
        elif data['cmd'] == 'activate':
            self.active = True
            if self.xbt_current_price != None:
                self.xbt_initial_price = self.xbt_current_price
    def process_data(self, data):
        if 'btc_usd_24hr' in data and 'last' in data['btc_usd_24hr']:
            self.xbt_current_price = float(data['btc_usd_24hr']['last'])
            if self.xbt_initial_price == None:
                self.debug("setting price to %f" % self.xbt_current_price)
                self.xbt_initial_price = self.xbt_current_price
        if (self.xbt_current_price < \
            self.xbt_initial_price * (1 - self.range) or
            self.xbt_current_price > \
            self.xbt_initial_price * (1 + self.range)):
            self.send_cancel()

if __name__ == "__main__":
    qm = StrategyXbtClose()
    qm.run()



#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License
import my_path
import time
import algobroker
import pprint

class StrategyAlert(algobroker.Strategy):
    def __init__(self):
        algobroker.Strategy.__init__(self, "strategy_alert",
                                     ['ticker_yahoo',
                                      'ticker_bitfutures',
                                      'ticker_bravenewcoin'])
        self.time_limits = {}
        self.state = {}
        self.prev_state = {}
        self.limits = {}
        self.quotes = {}
        self.maintainence = 60 * 30
    def test_limits(self):
        for i in self.limits.keys():
            if i in self.limits and i in self.quotes:
                limits = self.limits[i]
                if limits[0] != None and self.quotes[i] <= limits[0]:
                    self.state[i] = "low"
                elif limits[1] != None and self.quotes[i] >= limits[1]:
                    self.state[i] = "high"
                else:
                    self.state[i] = "ok"
    def send_notices(self):
        msg = ""
        for k, v in self.state.items():
            if k in self.prev_state:
                prev_state = self.prev_state[k]
            else:
                prev_state = "none"
            if v == "high" or v == "low":
                if prev_state != v:
                    msg += "%s - %f - %s | " % (k, self.quotes[k],
                                                v)
        if msg != "":
            self.send_action({'cmd' : 'alert',
                              'type' : 'sms',
                              'to' : 'trader1',
                              'text' : msg})
        for k, v in self.state.items():
            self.prev_state[k] = v
    def test(self):
        work_message = { 'cmd' : 'log',
                         'item' : 'hello' }
        self.send_action(work_message)
        work_message = { 'cmd' : 'alert',
                         'type' : 'sms',
                         'to'  : 'trader1',
                         'text' : 'hello and happy trading' }
        self.send_action(work_message)
    def process_control(self, data):
        algobroker.Strategy.process_control(self, data)
        if data['cmd'] == "set":
            if 'limits' in data:
                self.debug("setting limits")
                self.debug(pprint.pformat(data))
                self.limits = data['limits']
    def process_data(self, data):
        self.debug("running alert loop")
        self.debug(data)
        for k, v in data.items():
            if 'last' in v:
                self.quotes[k] = float(v['last'])
        self.test_limits()
        self.send_notices()

if __name__ == "__main__":
    qm = StrategyAlert()
    qm.run()



#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import pprint
import algobroker
from algobroker import Broker
import os
import subprocess

def is_exe(fpath):
    return os.path.exists(fpath) and os.access(fpath, os.X_OK)

class BrokerDeskAlert(Broker):
    def __init__(self):
        Broker.__init__(self, "broker_desk_alert")
        my_dir = os.path.dirname(os.path.realpath(__file__))
        self.media_dir = os.path.join(my_dir, "static")
        self.alerts = ["low", "high"]
        self.repeat = 5
        self.player = "/bin/play"
    def alert(self, alert):
        if not is_exe("/bin/play"):
            print("sox not installed")
        else:
            for i in range(self.repeat):
                subprocess.call(["/bin/play", "-q",
                                 os.path.join(self.media_dir,
                                              alert + ".ogg")])
    def process_data(self, data):
        if (data['cmd'] == "alert" and \
            data['type'] == 'desk'):
            if (data['alert'] in self.alerts):
                self.alert(data['alert'])
    def process_control(self, data):
        algobroker.Broker.process_control(self, data)

if __name__ == "__main__":
    bp = BrokerDeskAlert()
    bp.run()




#!/usr/bin/python3
# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

import my_path
import time
import algobroker
import pprint
from yahoo_finance import Share

class YahooTicker(algobroker.Ticker):
    def __init__(self):
        algobroker.Ticker.__init__(self, "ticker_yahoo")
        self.time_limits = {}
        self.state = {}
        self.prev_state = {}
        self.assets = []
    def get_quotes(self):
        self.debug("getting quotes")
        try:
            for i in self.assets:
                yahoo = Share(i)
                self.quotes[i] = {
                    "source" : "yahoo",
                    "last" : float(yahoo.get_price())
                    }
        except OSError:
            self.error("Network Error")
    def process_control(self, data):
        algobroker.Ticker.process_control(self, data)
        if data['cmd'] == "set":
            if 'assets' in data:
                self.debug("setting asset list")
                self.debug(pprint.pformat(data['assets']))
                self.assets = data['assets']

if __name__ == "__main__":
    yq = YahooTicker()
    yq.run()



# Copyright (C) 2015 Bitquant Research Laboratories (Asia) Limited
# Released under the Simplified BSD License

from setuptools import (
    setup,
    find_packages,
    )

setup(
    name="algobroker",
    version = "0.0.9",
    author="Joseph C Wang",
    author_email='joequant@gmail.com',
    url="https://github.com/joequant/algobroker",
    description="Algorithmic trading broker",
    long_description="""Algobroker is an interface to trading and events""",
    license="BSD",
    packages=['algobroker'],
    install_requires = ['pyzmq',
                        'msgpack-python',
                        'plivo',
                        'numpy',
                        'scipy',
                        'python-dateutil',
                        'yahoo_finance',
                        'requests',
                        'ib-api',
                        'gevent',
                        'simplejson',
                        'cryptoexchange >= 0.0.6'],
    scripts = ['start-algo.sh', 'stop-algo.sh'],
    use_2to3 = True
)
                                
